#ifndef _GLUGG_
#define _GLUGG_

// vec3 must be defined!
// I could declare _vec3 or whatever but then we would get typecasts all over the place.
#include "VectorUtils3.h"

#ifdef __cplusplus
extern "C" {
#endif

// GLUGG interface
// GLUGG is the OpenGL Utilties for Geometry Generation
// By Ingemar Ragnemalm 2013-2021
// Version 1.2.
// 1.2 adds gluggEndDisposable, gluggDisposeModel and renames gluggDispose to gluggDisposeData.

// Structure used for disposing of models created by gluggEndDisposable
typedef struct gluggModel
{
	int useDrawElements;
	GLuint vertexArrayObjID;
	GLuint vertexBufferObjID;
	GLuint normalsBufferObjID;
	GLuint colorsBufferObjID;
	GLuint texCoordBufferObjID;
	GLuint indexBufferObjID;
	int count;
} gluggModel;

void gluggBegin(int theMode);
void gluggVertex(float x, float y, float z);
void gluggVertexv(vec3 p);
void gluggNormal(float x, float y, float z);
void gluggNormalv(vec3 n);
void gluggTexCoord(float s, float t);
void gluggColor(float r, float g, float b); // New in version 1.1
void gluggColorv(vec3 c);
GLuint gluggEnd(int *count, GLuint program, char optimize);
void gluggDisposeData();

// New calls for disposing on-device data (i.e. when re-generating a model)
gluggModel gluggBuildModel(char optimize);
void gluggDisposeModel(gluggModel m);
void gluggDrawModel(gluggModel m, GLuint program);

void gluggMode(int newMode);
enum
{
	GLUGG_TRIANGLES = 0,
	GLUGG_QUADS,
	GLUGG_TRIANGLE_FAN,
	GLUGG_TRIANGLE_STRIP
};

void gluggBuildBezier(vec3 *vertices, int *indices, float step);
//GLint gluggBuildBezierPatchModel(vec3 *vertices, int *indices, int startPatch, int endPatch, int *count, GLuint program, float step);
gluggModel gluggBuildBezierPatchModel(vec3 *vertices, int *indices, int startPatch, int endPatch, float step);

// Inspect internal variables (use before gluggEnd, which clears them!)
GLfloat *gluggGetVertexArray();
GLfloat *gluggGetNormalsArray();
GLfloat *gluggGetColorsArray();
GLfloat *gluggGetTexCoordArray();
GLuint *gluggGetIndexArray();
int gluggGetNumVertices();
int gluggGetNumIndices();
void gluggArraysToElements(); // Needed before gluggGetNumIndices

void gluggSetPositionName(char *name);
void gluggSetNormalName(char *name);
void gluggSetColorName(char *name);
void gluggSetTexCoordName(char *name);

// Matrix stack support
void gluggPushMatrix();
void gluggPullMatrix();
void gluggPopMatrix();
void gluggTranslate(GLfloat x, GLfloat y, GLfloat z);
void gluggRotate(GLfloat angle, GLfloat x, GLfloat y, GLfloat z);
void gluggScale(GLfloat x, GLfloat y, GLfloat z);
mat4 gluggCurrentMatrix();
void gluggSetMatrix(mat4 m);
void gluggMultMatrix(mat4 m);


#ifdef __cplusplus
}
#endif

#endif
