// This demo calls the gluggShapes unit using reusable code for generating
// shapes originally from glut_shapes.
// New version 2002-03-19 for GLUGG 1.3

// uses framework Cocoa
// uses framework OpenGL
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "glugg.h"
#include "gluggShapes.h"

#ifndef M_PI
#define M_PI 3.14159265
#endif

static long count = 0;
static long limit = -1;
static float startTime;

static GLfloat view_rotx = 20.0, view_roty = 30.0, view_rotz = 0.0;
static gluggModel shape1, shape2, shape3, shape4, shape5, shape6; // Was display lists, now are VAOs
static GLfloat angle = 0.0;

//int shape1Triangles=0, shape2Triangles=0, shape3Triangles=0, shape4Triangles=0, shape5Triangles=0, shape6Triangles=0;

GLuint program; // Shader
mat4 projectionMatrix;

static GLfloat lightPos[4] = { 5.0, 5.0, 10.0, 0.0 };

static GLfloat red[4] = { 0.8, 0.1, 0.0, 1.0 };
static GLfloat green[4] = { 0.0, 0.8, 0.2, 1.0 };
static GLfloat blue[4] = { 0.2, 0.2, 1.0, 1.0 };
static GLfloat yellow[4] = { 1, 1, 0.2, 1.0 };
static GLfloat magenta[4] = { 1.0, 0.2, 1.0, 1.0 };
static GLfloat cyan[4] = { 0, 1, 1, 1.0 };

void draw(void)
{
//	mat4 rx, ry, rz, t, shapeRot;
	mat4 globalR, m;

  angle += 2.0;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
// GLOBAL ROTATION
	globalR = Mult(T(0, 0, -40.0), Mult(Rx(view_rotx*M_PI/180), Mult(Ry(-view_roty*M_PI/180), Rz(view_rotz*M_PI/180))));
	
// shape 1
	m = Mult(globalR, Mult(T(-4.0, -4.0, 0.0), Mult(Mult(Rx(angle/100), Rz(angle*M_PI/180)), S(0.5, 0.5, 0.5))));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, red);
	gluggDrawModel(shape1, program);
//	glBindVertexArray(shape1);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape1Triangles);

// shape 2
	m = Mult(globalR, Mult(T(3.1, -3.0, 0.0), Mult(Mult(Rx(angle/100), Rz((-2.0 * angle - 9.0)*M_PI/180)), S(1,1,1))));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, green);
	gluggDrawModel(shape2, program);
//	glBindVertexArray(shape2);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape2Triangles);

// shape 3
	m = Mult(globalR, Mult(T(-4.1, 4.2, 0.0), Mult(Mult(Ry(angle/100), Rz((-2.0 * angle - 25.0)*M_PI/180)), S(1,1,1))));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, blue);
	gluggDrawModel(shape3, program);
//	glBindVertexArray(shape3);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape3Triangles);
	
// shape 4
	m = Mult(globalR, Mult(T(3.1, 4.2, 0.0), Rz((-2.0 * angle - 25.0)*M_PI/180)));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, yellow);
	gluggDrawModel(shape4, program);
//	glBindVertexArray(shape4);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape4Triangles);
	
// shape 5
	m = Mult(globalR, Mult(T(0, 4.2, 0.0), Rz((-2.0 * angle - 25.0)*M_PI/180)));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, magenta);
	gluggDrawModel(shape5, program);
//	glBindVertexArray(shape5);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape5Triangles);
	
// shape 6
	m = Mult(globalR, Mult(T(0.1, 0, 0.0), Mult(Ry(angle / 50.0), Rx((-2.0 * angle)*M_PI/180))));
	
	glUniformMatrix4fv(glGetUniformLocation(program, "modelviewMatrix"), 1, GL_TRUE, m.m);
	glUniform4fv(glGetUniformLocation(program, "color"), 1, cyan);
	gluggDrawModel(shape6, program);
//	glBindVertexArray(shape6);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, shape6Triangles);
	
	glutSwapBuffers();
}

/* change view angle, exit upon ESC */
/* ARGSUSED1 */
static void
key(unsigned char k, int x, int y)
{
  switch (k)
  {
  case 'z':
    view_rotz += 5.0;
    break;
  case 'Z':
    view_rotz -= 5.0;
    break;
  case 27:  /* Escape */
    exit(0);
    break;
  default:
    return;
  }
  glutPostRedisplay();
}

/* change view angle */
/* ARGSUSED1 */
static void
special(unsigned char k, int x, int y)
{
  switch (k)
  {
  case GLUT_KEY_UP:
    view_rotx += 5.0;
    break;
  case GLUT_KEY_DOWN:
    view_rotx -= 5.0;
    break;
  case GLUT_KEY_LEFT:
    view_roty += 5.0;
    break;
  case GLUT_KEY_RIGHT:
    view_roty -= 5.0;
    break;
  default:
    return;
  }
  glutPostRedisplay();
}

/* new window size or exposure */
// This will give us fixed proportions instead of scaling, which is the default.
static void
reshape(int width, int height)
{
	GLfloat h = (GLfloat) height / (GLfloat) width;

	glViewport(0, 0, (GLint) width, (GLint) height);
	projectionMatrix = frustum(-1.0, 1.0, -h, h, 6.0, 60.0);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
}


static void
init(void)
{
	glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);

	projectionMatrix = frustum(-1.0, 1.0, -1.0, 1.0, 6.0, 60.0);

	program = loadShaders("shapes.vert", "shapes.frag");
	glUniform4fv(glGetUniformLocation(program, "pos"), 1, lightPos);
	glUniformMatrix4fv(glGetUniformLocation(program, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	
	/* make the shapes */
	// This now needs three calls each instead of one. The advantage is that we can record multiple shapes into one.
	gluggBegin(0);
	gluggTetrahedron(2.0);
	shape1 = gluggBuildModel(0);
	gluggBegin(0);
	gluggIcosahedron(2.0);
	shape2 = gluggBuildModel(0);
	gluggBegin(0);
	gluggOctahedron(2.0);
	shape3 = gluggBuildModel(0);
	
	gluggBegin(0);
	gluggDodecahedron(1.0);
	shape4 = gluggBuildModel(0);
	gluggBegin(0);
	gluggCube(1.5);
	shape5 = gluggBuildModel(0);
	gluggBegin(0);
	gluggDoughnut(0.7, 1.5, 20, 20);
	shape6 = gluggBuildModel(0);

	startTime = glutGet(GLUT_ELAPSED_TIME) / 1000.0;
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);

	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitContextVersion(3, 2);

	glutCreateWindow("GLUGG shapes (using gluggShapes, based on glut_shapes)");
	init();

	glutDisplayFunc(draw);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key);
	glutSpecialFunc(special);
	glutRepeatingTimer(15);

	glutMainLoop();
	return 0;             /* ANSI C requires main to return int. */
}
