// GLUGG snowman
// A demo of using the matrix stack and building from multiple parts

// Cut-down version of the "newsnowman" without the billboards
// Based on old demos from Lighthouse 3D

// Written for GLUGG 1.1, using the color support and gluggShapes.
// 1.3 version, using gluggBuildModel and gluggDrawModel.

	//uses framework Cocoa
	//uses framework OpenGL
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "LoadTGA.h"
#include "glugg.h"
#include "gluggShapes.h"

mat4 projectionMatrix;

GLuint grasstex;

// Reference to shader programs
GLuint phongShader, texShader;

gluggModel MakeSnowman()
{
	gluggSetPositionName("inPosition");
	gluggSetNormalName("inNormal");
	gluggSetColorName("inColor");
	gluggSetTexCoordName("inTexCoord");

	gluggBegin(GLUGG_TRIANGLES);

	gluggColor(1.0f, 1.0f, 1.0f);

// Draw Body	
	gluggTranslate(0.0f ,0.75f, 0.0f);
	gluggSphere(20,20, 0.75f);

// Draw Head
	gluggTranslate(0.0f, 0.95f, 0.0f);
	gluggSphere(20,20, 0.25f);

// Draw Eyes
//	printMat4(gluggCurrentMatrix());
	gluggPushMatrix();
	gluggColor(0.0f,0.0f,0.0f);
	gluggTranslate(0.05f, 0.10f, 0.18f);
//	printMat4(gluggCurrentMatrix());
	gluggSphere(10,10, 0.05f);
	gluggTranslate(-0.1f, 0.0f, 0.0f);
	gluggSphere(10,10, 0.05f);
	gluggPopMatrix();

// Draw Nose
	gluggColor(1.0f, 0.5f , 0.5f);
	gluggRotate(M_PI/2.0,1.0f, 0.0f, 0.0f);
	gluggCone(10, 0.5, 0.08f);

	return gluggBuildModel(0);
}

gluggModel MakeFloor()
{
	gluggSetPositionName("inPosition");
	gluggSetNormalName("inNormal");
	gluggSetColorName("inColor");
	gluggSetTexCoordName("inTexCoord");

	gluggBegin(GLUGG_QUADS);

	gluggNormal(0.0f, 1.0f, 0.0f);
	gluggTexCoord(500.0f, 500.0f);
	gluggVertex(-200, 0, -200);
	gluggTexCoord(0.0f, 500.0f);
	gluggVertex(200, 0, -200);
	gluggTexCoord(0.0f, 0.0f);
	gluggVertex(200, 0, 200);
	gluggTexCoord(500.0f, 0.0f);
	gluggVertex(-200, 0, 200);

// With GLUGG_TRIANGLES you would do like this:
//	gluggNormal(0.0f, 1.0f, 0.0f);
//	gluggTexCoord(500.0f, 500.0f);
//	gluggVertex(-200, 0, -200);
//	gluggTexCoord(0.0f, 500.0f);
//	gluggVertex(200, 0, -200);
//	gluggTexCoord(0.0f, 0.0f);
//	gluggVertex(200, 0, 200);
//	
//	gluggTexCoord(500.0f, 500.0f);
//	gluggVertex(-200, 0, -200);
//	gluggTexCoord(0.0f, 0.0f);
//	gluggVertex(200, 0, 200);
//	gluggTexCoord(0.0f, 500.0f);
//	gluggVertex(-200, 0, 200);

	return gluggBuildModel(0);
}

gluggModel snowman;
gluggModel theFloor;

void reshape(int w, int h)
{
    glViewport(0, 0, w, h);

	// Set the clipping volume
	projectionMatrix = perspective(45,1.0f*w/h,1,1000);
	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(texShader);
	glUniformMatrix4fv(glGetUniformLocation(texShader, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
}

void init(void)
{
	// GL inits
	glClearColor(0.2,0.2,0.5,0);
	glEnable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	printError("GL inits");

	projectionMatrix = frustum(-0.1, 0.1, -0.1, 0.1, 0.2, 300.0);

	// Load and compile shader
	phongShader = loadShaders("phong.vert", "phong.frag");
	texShader = loadShaders("textured.vert", "textured.frag");
	printError("init shader");

// Important! The shader we upload to must be active!
	glUseProgram(phongShader);
	glUniformMatrix4fv(glGetUniformLocation(phongShader, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);
	glUseProgram(texShader);
	glUniformMatrix4fv(glGetUniformLocation(texShader, "projectionMatrix"), 1, GL_TRUE, projectionMatrix.m);

	glUniform1i(glGetUniformLocation(texShader, "tex"), 0); // Texture unit 0

	LoadTGATextureSimple("grass.tga", &grasstex);
	glBindTexture(GL_TEXTURE_2D, grasstex);
	glTexParameteri(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_S,	GL_REPEAT);
	glTexParameteri(GL_TEXTURE_2D,	GL_TEXTURE_WRAP_T,	GL_REPEAT);

	snowman = MakeSnowman();
	theFloor = MakeFloor();
		
	printError("init arrays");
}

GLfloat a = 0.0;
vec3 campos = {0, 1.5, 10};
vec3 forward = {0, 0, -4};
vec3 up = {0, 1, 0};

void DrawSnowman(float x, float y, float z, mat4 worldToView)
{
	// Snowman
	glUseProgram(phongShader);
    mat4 m = Mult(worldToView, T(x, y, z));
    glUniformMatrix4fv(glGetUniformLocation(phongShader, "modelviewMatrix"), 1, GL_TRUE, m.m);
	gluggDrawModel(snowman, phongShader);
//	glBindVertexArray(snowman);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, snowmancount);
}

void DrawFloor(mat4 worldToView)
{
	// Snowman
	glUseProgram(texShader);
    glUniformMatrix4fv(glGetUniformLocation(texShader, "modelviewMatrix"), 1, GL_TRUE, worldToView.m);
	gluggDrawModel(theFloor, texShader);
//	glBindVertexArray(theFloor);	// Select VAO
//	glDrawArrays(GL_TRIANGLES, 0, floorcount);
}

void display(void)
{
	int i, j;
	
	printError("pre display");

	// clear the screen
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	mat4 worldToView, m; // m1, m2, m, tr, scale;
	
	if (glutKeyIsDown('a'))
		forward = MultMat3Vec3(mat4tomat3(Ry(0.05)), forward);
	if (glutKeyIsDown('d'))
		forward = MultMat3Vec3(mat4tomat3(Ry(-0.05)), forward);
	if (glutKeyIsDown('w'))
		campos = VectorAdd(campos, ScalarMult(forward, 0.1));
	if (glutKeyIsDown('s'))
		campos = VectorSub(campos, ScalarMult(forward, 0.1));
	if (glutKeyIsDown('q'))
	{
		vec3 side = CrossProduct(forward, SetVector(0,1,0));
		campos = VectorSub(campos, ScalarMult(side, 0.05));
	}
	if (glutKeyIsDown('e'))
	{
		vec3 side = CrossProduct(forward, SetVector(0,1,0));
		campos = VectorAdd(campos, ScalarMult(side, 0.05));
	}

	// Move up/down
	if (glutKeyIsDown('z'))
		campos = VectorAdd(campos, ScalarMult(SetVector(0,1,0), 0.01));
	if (glutKeyIsDown('c'))
		campos = VectorSub(campos, ScalarMult(SetVector(0,1,0), 0.01));

	// NOTE: Looking up and down is done by making a side vector and rotation around arbitrary axis!
	if (glutKeyIsDown('+'))
	{
		vec3 side = CrossProduct(forward, SetVector(0,1,0));
		mat4 m = ArbRotate(side, 0.05);
		forward = MultMat3Vec3(mat4tomat3(m), forward);
	}
	if (glutKeyIsDown('-'))
	{
		vec3 side = CrossProduct(forward, SetVector(0,1,0));
		mat4 m = ArbRotate(side, -0.05);
		forward = MultMat3Vec3(mat4tomat3(m), forward);
	}	

	worldToView = lookAtv(campos, VectorAdd(campos, forward), up);
	
	a += 0.1;

	glBindTexture(GL_TEXTURE_2D, grasstex);
	
	DrawFloor(worldToView);

	DrawSnowman(0, 0, 0, worldToView);
	DrawSnowman(5, 0, 0, worldToView);
	DrawSnowman(0, 0, 5, worldToView);
	DrawSnowman(0, 0, -5, worldToView);
	DrawSnowman(-5, 0, 0, worldToView);
	
	printError("display");
	
	glutSwapBuffers();
}

void keys(unsigned char key, int x, int y)
{
	switch (key)
	{
		case ' ':
			forward.y = 0;
			forward = ScalarMult(normalize(forward), 4.0);
			break;
	}
}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	glutInitContextVersion(3, 2);
	glutInitDisplayMode(GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGBA);
	glutInitWindowPosition(100,100);
	glutInitWindowSize(640,360);
	glutCreateWindow ("GLUGG Snowman example");
	glutRepeatingTimerFunc(20);
	glutDisplayFunc(display); 
	glutKeyboardFunc(keys);
	glutReshapeFunc(reshape);
	init ();
	glutMainLoop();
}
