// Demo of the different recording modes, an extension of the gluggTriangle demos.
// By Ingemar 2021. GLUGG 1.3 version 2022.

// uses framework Cocoa
// uses framework OpenGL
#include "MicroGlut.h"
#include "GL_utilities.h"
#include "VectorUtils3.h"
#include "glugg.h"

GLuint program; // Shader

gluggModel triangles, quads, fan, strip;

void draw(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_CULL_FACE);

	gluggDrawModel(triangles, program);
	gluggDrawModel(quads, program);
	gluggDrawModel(fan, program);
	gluggDrawModel(strip, program);

	glutSwapBuffers();
}


void init(void)
{
	program = loadShaders("color.vert", "color.frag");
	
	// For this demo, I make one shape (buffer) in each mode.
	// Note that it is legal to switch mode withon one shape.
	
	glDisable(GL_CULL_FACE);
	
	/* make some triangles */
	gluggBegin(GLUGG_TRIANGLES);

	gluggNormal(1, 0, 0); // Red
	
	gluggVertex(0.1, 0.6,0);
	gluggVertex(0.4, 0.7,0);
	gluggVertex(0.3, 0.8,0);

	gluggVertex(0.6, 0.9,0);
	gluggVertex(0.7, 0.7,0);
	gluggVertex(0.9, 0.8,0);
	
	gluggVertex(0.5, 0.5,0);
	gluggVertex(0.4, 0.2,0);
	gluggVertex(0.7, 0.3,0);
	
	triangles = gluggBuildModel(0);

	// Make some quads
	gluggBegin(GLUGG_QUADS);

	gluggNormal(0, 1, 0); // Green

	gluggVertex(0.2, -0.5, 0);
	gluggVertex(0.4, -0.4, 0);
	gluggVertex(0.3, -0.2, 0);
	gluggVertex(0.1, -0.3, 0);

	gluggVertex(0.5, -0.9, 0);
	gluggVertex(0.6, -0.7, 0);
	gluggVertex(0.4, -0.6, 0);
	gluggVertex(0.3, -0.8, 0);

	gluggVertex(0.5, -0.5, 0);
	gluggVertex(0.7, -0.5, 0);
	gluggVertex(0.7, -0.3, 0);
	gluggVertex(0.5, -0.3, 0);

	quads = gluggBuildModel(0);

	// Make a fan
	gluggBegin(GLUGG_TRIANGLE_FAN);

	gluggNormal(0, 1, 1); // Cyan

	gluggVertex(-0.5, -0.5, 0); // Center

	gluggVertex(-0.7, -0.3, 0);
	gluggVertex(-0.8, -0.5, 0);
	gluggVertex(-0.7, -0.7, 0);
	gluggVertex(-0.5, -0.8, 0);
	gluggVertex(-0.3, -0.7, 0);
	gluggVertex(-0.2, -0.5, 0);
	gluggVertex(-0.3, -0.3, 0);
	gluggVertex(-0.5, -0.2, 0); // Intentionally missing one vertex!

	fan = gluggBuildModel(0);

	// Make a strip
	gluggBegin(GLUGG_TRIANGLE_STRIP);

	gluggNormal(1, 1, 0); // Yellow

	gluggVertex(-0.3, 0.1, 0);
	gluggVertex(-0.2, 0.3, 0);
	gluggVertex(-0.5, 0.3, 0);
	gluggVertex(-0.3, 0.4, 0);
	gluggVertex(-0.5, 0.5, 0);
	gluggVertex(-0.3, 0.6, 0);
	gluggVertex(-0.7, 0.7, 0);
	gluggVertex(-0.5, 0.8, 0);

	strip = gluggBuildModel(0);

}

int main(int argc, char *argv[])
{
	glutInit(&argc, argv);
	
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE);
	glutInitContextVersion(3, 2);

	glutCreateWindow("GLUGG Modes");
	init();
	glutDisplayFunc(draw);

	glutMainLoop();
	return 0;
}
